#!/usr/bin/python

import os
import re
import sys

import click
from jmb.tools.functions import execute

def transform_file(filename):
    with open(filename) as f:
        txt = f.read()

    with open(filename, 'w') as f:
        f.write(transform_content(txt, filename))


def transform_content(txt, filename):
    if 'from __future__' not in txt and not filename.endswith('/setup.py'):
        txt = add_import('from __future__ import unicode_literals\n', txt)
    substitutions = {
        'smart_str': 	'smart_bytes',
        'smart_unicode': 	'smart_text',
        'force_unicode': 	'force_text',
        'EscapeString': 	'EscapeBytes',
        'EscapeUnicode': 	'EscapeText',
        'SafeString': 	'SafeBytes',
        'SafeUnicode': 	'SafeText',
        'basestring': 'six.string_types',
        'unicode': 'six.text_type',
        #'str': 'bytes', # risulta in doppia sostituzione unicode => (2to3) => bytes
        '__unicode__': '__str__',
        'StringIO': 'six.StringIO',
        'file\(': 'open(',
        }
    for old_name, new_name in substitutions.items():
        txt = re.sub(r'\b{}\b'.format(old_name), new_name, txt)

    if filename.endswith('/models.py'):
        txt = transform_models(txt)

    if 'six' in txt:
        txt = add_import('from django.utils import six', txt, 'django')

    return txt

def add_import(import_stmt, txt, before=''):
    "Add import statement in txt"
    if import_stmt not in txt:
        txt = re.sub(
            '^(import.*{0}|from.*{0}.*import)'.format(before),
            '{}\n\\1'.format(import_stmt), txt, flags=re.M, count=1)
    return txt


def transform_models(txt):
    if 'python_2_unicode_compatible' in txt:
        return txt
    txt = re.sub('^(class .*)', r'@python_2_unicode_compatible\n\1', txt, flags=re.M)
    txt = add_import('from django.utils.encoding import python_2_unicode_compatible', txt, 'django')
    txt = re.sub('__unicode__', '__str__', txt)
    return txt

def loop_all_files(start_dir):
    for root, dirs, files in os.walk(start_dir):
        for f in files:
            if f.endswith('.py'):
                filename = os.path.join(root, f)
                transform_file(filename)


## PROGRAM
@click.command('23')
@click.argument('repo', required=True)
def main(repo):
    """Clone a repository, apply all migrations to port
     to py3 in a py2 compatible way"""
    #repo = sys.argv[1]
    new_dir = os.path.basename(repo.rstrip('/'))
    new_dir2 = "{}_2".format(new_dir)

    execute("hg clone {}".format(repo))
    print("hg clone {}".format(repo))
    execute("hg clone {} {}".format(repo, new_dir2))
    print("hg clone {} {}".format(repo, new_dir2))
    os.chdir(new_dir)
    print("cd {}".format(new_dir))
    cmd = "2to3 --nobackups -f all -f idioms -f set_literal -f ws_comma -x basestring -x callable --no-diffs -w ."
    print(cmd)

    ret_code, err, out = execute(cmd)
    if err:
        print(err)
    execute(['hg', 'qnew', '-f', '2to3', '-m', 'Porting to Python3: step 2to3'] )

    loop_all_files('.')

    execute(['hg', 'qnew', '-f', '2e3', '-m', 'Porting to Python3: step jmb-2e3'] )
    execute(['hg', 'qnew', '-f', 'manual-rev', '-m', 'Porting to Python3: manual revision'] )

    ret, err, out = execute("hg qseries")
    print(out)

    print("""
    meld {new_dir} {new_dir2}
      ... e poi:
             cd {new_dir}
             hg qrefresh
             hg qfinish -a
             hg push
             cd {repo}
             hg up
    """.format(**locals()))
    print("cd {new_dir}; hg qrefresh; hg qfinish -a; hg push; cd {repo}; hg up".format(**locals()))
